#include "kernel.h"
#include "kernel_id.h"
#include "ecrobot_interface.h"
#include "lift.h"
#include "myrobot.h"


static  int error, error_prev, error_sum;

//リフト位置の初期化
void lift_init(){

	int data;
	int prev_data=999;

	error=0;
	error_prev=0;
	error_sum=0;

	while(1){
		data=nxt_motor_get_count(LIFT_MOTOR);
		if(data == prev_data) break;
		nxt_motor_set_speed(LIFT_MOTOR,-60,0);	
		systick_wait_ms(100);
		prev_data=data;
	}
	nxt_motor_set_speed(LIFT_MOTOR,0,1);		
	nxt_motor_set_count(LIFT_MOTOR,0);
}

//エラー値をリセットする
void lift_error_reset(){

	error=0;
	error_prev=0;
	error_prev=0;

}

//指定角までリフトを下げる
void lift_down(int angle){

	while(nxt_motor_get_count(LIFT_MOTOR) < angle){	
		nxt_motor_set_speed(LIFT_MOTOR,50,0);	
		systick_wait_ms(CNTL_TIME);
	}
	nxt_motor_set_speed(LIFT_MOTOR,0,1);		
}


//指定角までリフトを上げる
void lift_up(int angle){

	while(nxt_motor_get_count(LIFT_MOTOR) > angle){	
		nxt_motor_set_speed(LIFT_MOTOR,-70,0);
		systick_wait_ms(CNTL_TIME);
	}
	nxt_motor_set_speed(LIFT_MOTOR,0,1);		
}

//修復材を保持した状態でリフトを指定した角度にをゆっくり移動させる。
void lift_move_hold(int angle){

	int count=0;
	int st_angle;
	int target_angle;
	float dif_angle;
	int loop_num = 200; // このループ回数でリフトを所定の角度に移動させる

	st_angle=nxt_motor_get_count(LIFT_MOTOR);
	dif_angle= (float)(angle - st_angle)/loop_num; // 10msec *200 step = 2sec で目標値

	lift_error_reset();

	//while(nxt_motor_get_count(LIFT_MOTOR) > angle){	/* モータの回転角度が指定角度以下が真 */
	while(count < loop_num){	/* モータの回転角度が指定角度以下が真 */
		target_angle= dif_angle*count + st_angle;
		lift_hold(target_angle);
		systick_wait_ms(CNTL_TIME);
		count++;
	}
	//nxt_motor_set_speed(LIFT_MOTOR,0,1);		
}

//リフト角を保持する。
void lift_hold(int angle){
	static int force=0; // -30;
	static float Kp=10.0;
	static float Kd=0.0;
	static float Ki=0.04;

	error = angle - nxt_motor_get_count(LIFT_MOTOR) ;
	
	force = Kp*error + Ki*error_sum + Kd*(error - error_prev);
	if(force > 100) force=100;
	if(force < -100) force=-100;

	error_prev=error;
	error_sum+=error;

	nxt_motor_set_speed(LIFT_MOTOR,force,1);		

	//display_clear(0);
	display_goto_xy(0, 1);
	display_string("error=");
	display_int(error, 1);
	display_goto_xy(0, 2);
	display_string("force=");
	display_int(force, 1);
	display_update();

}

//指定時間リフト角を保持する。
void lift_hold_time(int angle, int time){

	static int force=0; // -30;
	static float Kp=10.0;
	static float Kd=0.0;
	static float Ki=0.04;

	int count=0;
	int max_count = time/CNTL_TIME;

	while(count < max_count){ 

		error = angle - nxt_motor_get_count(LIFT_MOTOR) ;
	
		force = Kp*error + Ki*error_sum + Kd*(error - error_prev);
		if(force > 100) force=100;
		if(force < -100) force=-100;

		error_prev=error;
		error_sum+=error;
		nxt_motor_set_speed(LIFT_MOTOR,force,1);		

		count++;
		systick_wait_ms(CNTL_TIME);

	}

}

